import os
from fastapi import APIRouter
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions
from ..database.mycosmosdb import callConnect
from ..configs.config import NODE_DEV
from typing import List
import logging


NODE_DEV = os.getenv("NODE_DEV")


if NODE_DEV == "production":
    MY_DOMAIN = "https://metricplatformapi.com/"
    MY_HOME = "https://www.metric-platform.com/"
elif NODE_DEV == "test":
    MY_DOMAIN = "https://metricplatform-staging.azurewebsites.net/"
    MY_HOME = "https://metricplatform-info-metric-esgc-metric-dev.vercel.app/"
else:
    MY_DOMAIN = "http://localhost:8000/"
    MY_HOME = "http://localhost:3000/"





class Reqs(BaseModel):
    id: str

class Goals(BaseModel):
    name: str
    id: str
    description: str
    targetmetrics: List[str]
    timeline: str
    relatedprojects: List[str]
    companies: List[str]
class EGoals(BaseModel):
    ename: str
    edescription: str
    name: str
    id: str
    description: str
    targetmetrics: List[str]
    timeline: str
    relatedprojects: List[str]
    companies: List[str]
class Projects(BaseModel):
    name: str
    id: str
    description: str
    targetmetrics: List[str]
    timeline: str
    relatedgoals: List[str]
    companies: List[str]
class EProjects(BaseModel):
    ename: str
    edescription: str
    name: str
    id: str
    description: str
    targetmetrics: List[str]
    timeline: str
    relatedgoals: List[str]
    companies: List[str]
class DProject(BaseModel):
    id: str
    name: str
    description: str
class InviteMember(BaseModel):
    oid: str
    city: str
    title: str
    email: str


def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response



goalprojectroute = APIRouter()


@goalprojectroute.post('/getgp')
def get_gp(reqs: Reqs):
    try:
        db_result = read_item(callConnect.container4, reqs.id, reqs.id)
        
    except exceptions.CosmosHttpResponseError as e:
        user = {
            "id": reqs.id,
            "goals": [],
            "projects": []
        }
        callConnect.container4.upsert_item(body=user)
        

    if db_result:
        if "goals" in db_result:
            if "projects" in db_result:
                return {"goals": db_result["goals"], "projects": db_result["projects"]}
            else:
                return {"goals": db_result["goals"]}
        else:
            if "projects" in db_result:
                return {"projects": db_result["projects"]}
    else:
        pass


@goalprojectroute.post('/newgoal')
async def add_goal(goals: Goals):
    ndate = f"{goals.timeline[5:7]}-{goals.timeline[8:10]}-{goals.timeline[:4]}"
    try:
        db_result = read_item(callConnect.container4, goals.id, goals.id)        

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /getgp container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            ngoals = db_result["goals"]
            ngoals.append({"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies})
            goal = {
                "goals": ngoals
            }
    
        else:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"]
            }
            ngoals = db_result["goals"]
            ngoals.append({"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies})
            goal = {
                "goals": ngoals
            }
    else:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "projects": db_result["projects"]
            }
            goal = {
                "goals": [
                    {"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies}
                ]
            }
    
        else:
            user = {
                "id": db_result["id"]
            }
            goal = {
                "goals": [
                    {
                        "name": goals.name,
                        "description": goals.description,
                        "targetmetrics": goals.targetmetrics,
                        "timeline": ndate,
                        "relatedprojects": goals.relatedprojects,
                        "companies": goals.companies
                    }
                ]
            }

    user.update(goal)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}


@goalprojectroute.post('/newproject')
async def add_project(projects: Projects):
    ndate = f"{projects.timeline[5:7]}-{projects.timeline[8:10]}-{projects.timeline[:4]}"
    try:
        db_result = read_item(callConnect.container4, projects.id, projects.id)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /newproject container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            nprojects = db_result["projects"]
            nprojects.append({"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies})
            project = {
                "projects": nprojects
            }
    
        else:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"]
            }
            nprojects = {"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies}
            project = {
                "projects": [nprojects]
            }
    else:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "projects": db_result["projects"]
            }
            nprojects = db_result["projects"]
            nprojects.append({"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies})
            project = {
                "projects": nprojects
            }
    
        else:
            user = {
                "id": db_result["id"]
            }
            nprojects = {"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies}
            project = {
                "projects": [nprojects]
            }

    user.update(project)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}


@goalprojectroute.post('/editgoal')
async def add_goal(goals: EGoals):
    ndate = f"{goals.timeline[5:7]}-{goals.timeline[8:10]}-{goals.timeline[:4]}"
    try:
        db_result = read_item(callConnect.container4, goals.id, goals.id)        

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /editgoal container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            ngoals = [z for z in db_result["goals"] if z["name"] != goals.ename if z["description"] != goals.edescription]
            ngoals.append({"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies})
            goal = {
                "goals": ngoals
            }
    
        else:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"]
            }
            ngoals = [z for z in db_result["goals"] if z["name"] != goals.ename if z["description"] != goals.edescription]
            ngoals.append({"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies})
            goal = {
                "goals": ngoals
            }
    else:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "projects": db_result["projects"]
            }
            goal = {
                "goals": [
                    {"name": goals.name, "description": goals.description, "targetmetrics": goals.targetmetrics, "timeline": ndate, "relatedprojects": goals.relatedprojects, "companies": goals.companies}
                ]
            }
    
        else:
            user = {
                "id": db_result["id"]
            }
            goal = {
                "goals": [
                    {
                        "name": goals.name,
                        "description": goals.description,
                        "targetmetrics": goals.targetmetrics,
                        "timeline": ndate,
                        "relatedprojects": goals.relatedprojects,
                        "companies": goals.companies
                    }
                ]
            }

    user.update(goal)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}

@goalprojectroute.post('/deletegoal')
async def add_goal(goals: DProject):
    try:
        db_result = read_item(callConnect.container4, goals.id, goals.id)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /deletegoal container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            ngoals = [z for z in db_result["goals"] if z["name"] != goals.name if z["description"] != goals.description]
            goal = {
                "goals": ngoals
            }
    
        else:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"]
            }
            ngoals = [z for z in db_result["goals"] if z["name"] != goals.name if z["description"] != goals.description]
            goal = {
                "goals": ngoals
            }

    user.update(goal)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}


@goalprojectroute.post('/editproject')
async def add_goal(projects: EProjects):
    ndate = f"{projects.timeline[5:7]}-{projects.timeline[8:10]}-{projects.timeline[:4]}"
    try:
        db_result = read_item(callConnect.container4, projects.id, projects.id)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /editproject container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            nprojects = [z for z in db_result["projects"] if z["name"] != projects.ename if z["description"] != projects.edescription]
            nprojects.append({"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies})
            project = {
                "projects": nprojects
            }
    else:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "projects": db_result["projects"]
            }
            nprojects = [z for z in db_result["projects"] if z["name"] != projects.ename if z["description"] != projects.edescription]
            nprojects.append({"name": projects.name, "description": projects.description, "targetmetrics": projects.targetmetrics, "timeline": ndate, "relatedgoals": projects.relatedgoals, "companies": projects.companies})
            project = {
                "projects": nprojects
            }

    user.update(project)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}


@goalprojectroute.post('/deleteproject')
async def add_goal(projects: DProject):
    try:
        db_result = read_item(callConnect.container4, projects.id, projects.id)

    except exceptions.CosmosHttpResponseError as e:
        logging.error('This is an error message for /deleteproject container4')
        return {"url": "/", "atype": False}

    if "goals" in db_result:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "goals": db_result["goals"],
                "projects": db_result["projects"]
            }
            nprojects = [z for z in db_result["projects"] if z["name"] != projects.name if z["description"] != projects.description]
            project = {
                "projects": nprojects
            }
    else:
        if "projects" in db_result:
            user = {
                "id": db_result["id"],
                "projects": db_result["projects"]
            }
            nprojects = [z for z in db_result["projects"] if z["name"] != projects.name if z["description"] != projects.description]
            project = {
                "projects": nprojects
            }

    user.update(project)
    callConnect.container4.upsert_item(body=user)
    return {"url": f"{MY_HOME}dashboard/goalsandprojects"}


